/**
 * @file main.c
 * @author sylvain.gaeremynck@parrot.com
 * @date 2009/07/01
 */
#include <ardrone_testing_tool.h>

//ARDroneLib
#include <ardrone_tool/ardrone_time.h>
#include <ardrone_tool/Navdata/ardrone_navdata_client.h>
#include <ardrone_tool/Control/ardrone_control.h>
#include <ardrone_tool/UI/ardrone_input.h>

//Common
#include <config.h>
#include <ardrone_api.h>
#include <string.h>
#include <stdio.h>

//VP_SDK
#include <ATcodec/ATcodec_api.h>
#include <VP_Os/vp_os_print.h>
#include <VP_Api/vp_api_thread_helper.h>
#include <VP_Os/vp_os_signal.h>

//Local project
#include <UI/gamepad.h>
#include <Video/video_stage.h>

//gtk support
#include <gtk/gtk.h>

#include <cv.h>
#include <highgui.h>

//ImageData
#define buffWidth 320
#define buffHeight 240

static int32_t exit_ihm_program = 1;

/*User interface global variables and defines*/

GtkWidget *window = NULL;
GtkImage *image = NULL;
GdkPixbuf *pixbuf = NULL;
GtkWidget *infolabel=NULL;

extern uint8_t*  pixbuf_data;
extern vp_os_mutex_t  video_update_lock;

/*variables to pass to the evaluation function*/
int boxwidth=30;
int boxheight=50;


/* Implementing Custom methods for the main function of an ARDrone application */

/* The delegate object calls this method during initialization of an ARDrone application */
C_RESULT ardrone_tool_init_custom(int argc, char **argv)
{
    /* Registering for a new device of game controller */
    ardrone_tool_input_add( &gamepad );

    /* Start all threads of your application */
    START_THREAD( video_stage, NULL );
    START_THREAD( userinterface, NULL );

    return C_OK;
}

/* The delegate object calls this method when the event loop exit */
C_RESULT ardrone_tool_shutdown_custom()
{
    /* Relinquish all threads of your application */
    JOIN_THREAD( video_stage );
    JOIN_THREAD( userinterface );

    /* Unregistering for the current device */
    ardrone_tool_input_remove( &gamepad );

    return C_OK;
}

/* The event loop calls this method for the exit condition */
bool_t ardrone_tool_exit()
{
    return exit_ihm_program == 0;
}

C_RESULT signal_exit()
{
    exit_ihm_program = 0;

    return C_OK;
}

/*User interface functions*/

gint delete_event( GtkWidget *widget,
                   GdkEvent  *event,
                   gpointer   data )
{
    gtk_main_quit ();
    signal_exit();
    return FALSE;
}

/*visualization window*/

void update_vision( void )
{
    vp_os_mutex_lock(&video_update_lock);
    char *pixbuf_to_draw;
    int i;
    int j;


    if( pixbuf_data != NULL )
    {
        pixbuf_to_draw=vp_os_malloc(320*240*3);

        vp_os_memset(pixbuf_to_draw,0,320*240*3);

        for(i=0; i<buffHeight; i++)
        {
            for(j=0; j<buffWidth; j++)
            {
                if(((j>=buffWidth/2-boxwidth)&&(j<=buffWidth/2+boxwidth)&&((i==buffHeight/2-boxheight)||(i==buffHeight/2+boxheight)))
                        ||((i>=buffHeight/2-boxheight)&&(i<=buffHeight/2+boxheight)&&((j==buffWidth/2-boxwidth)||(j==buffWidth/2+boxwidth))))
                {
                    pixbuf_to_draw[j*3+i*buffWidth*3]=255;
                    pixbuf_to_draw[j*3+i*buffWidth*3+1]=0;
                    pixbuf_to_draw[j*3+i*buffWidth*3+2]=0;
                }
                else
                {
                    pixbuf_to_draw[j*3+i*buffWidth*3]=pixbuf_data[j*3+i*buffWidth*3];
                    pixbuf_to_draw[j*3+i*buffWidth*3+1]=pixbuf_data[j*3+i*buffWidth*3+1];
                    pixbuf_to_draw[j*3+i*buffWidth*3+2]=pixbuf_data[j*3+i*buffWidth*3+2];
                }
            }
        }
        pixbuf = gdk_pixbuf_new_from_data(pixbuf_to_draw,
                                          GDK_COLORSPACE_RGB,
                                          FALSE,
                                          8,
                                          320,
                                          240,
                                          320*3,
                                          NULL,
                                          NULL);

        if( image != NULL )

        {
            gtk_image_set_from_pixbuf(image, pixbuf);
        }

        g_object_unref(pixbuf);
        vp_os_free(pixbuf_to_draw);
    }

    vp_os_mutex_unlock(&video_update_lock);

    gtk_widget_show_all( window );
}

gint incw(GtkWidget *widget, gpointer data)
{
    boxwidth+=3;
    return C_OK;
}

gint decw(GtkWidget *widget, gpointer data)
{
    boxwidth-=3;
    return C_OK;
}

gint inch(GtkWidget *widget, gpointer data)
{
    boxheight+=3;
    return C_OK;
}

gint dech(GtkWidget *widget, gpointer data)
{
    boxheight-=3;
    return C_OK;
}

void printResultsOnFile(int H, int S, int V, int isRobe)
{
	FILE *fout, *fin;
	int minHL, maxHL, minHH, maxHH, minS, minV, maxS, maxV;
	char* stringIn, *stringOut;

	fin=fopen("videoconfig.txt","r");
	fout=fopen("videoconfig2.txt","w");

	int const lengthOfLine=19;
	stringIn=(char*)vp_os_malloc(lengthOfLine*sizeof(char));
	stringOut=(char*)vp_os_malloc(lengthOfLine*sizeof(char));

	int const numberOfLinesInFile=14;
	
	minS=(S-20)>0?S-20:0;
	minV=(V-40)>0?V-40:0;
	maxS=(S+30)<255?S+30:255;
	maxV=(V+20)<255?V+20:255;
	minHL=H-15;
	maxHL=H+15;
	maxHH=maxHL;
	minHH=minHL;
	if (maxHH>255)
	{
		minHL=0;
		maxHL=maxHH-255;
		maxHH=255;
	}
	if (minHL<0)
	{
		minHH=minHL+255;
		maxHH=255;
		minHL=0;
	}
	int dirtyEscape=0;
	while (!feof(fin))
	{
		fscanf(fin,"%[^\n]\n",stringIn);
		//fgets(stringIn, lengthOfLine, fin);
		dirtyEscape++;
			if (isRobe)
			{
				if (stringIn[4]=='R')
				{
					//This line has to be modified
					if (stringIn[1]=='I')
					{
						//This is a Minimum Line
						switch (stringIn[3])
						{
							case 'H':
								sprintf(stringOut,"MINHROBE = %d;\n", minHH);
							break;
		
							case 'S':
								sprintf(stringOut,"MINSROBE = %d;\n", minS);
							break;
						
							case 'V':
								sprintf(stringOut,"MINVROBE = %d;\n", minV);
							break;
						}
					}
					else
					{
						//This is a Maximum Line
						switch (stringIn[3])
						{
							case 'H':
								sprintf(stringOut,"MAXHROBE = %d;\n", maxHH);
							break;
		
							case 'S':
								sprintf(stringOut,"MAXSROBE = %d;\n", maxS);
							break;
						
							case 'V':
								sprintf(stringOut,"MAXVROBE = %d;\n", maxV);
							break;
						}
					}
				}
				else
				{
					//This line goes straight in the output	
					strcpy(stringOut,stringIn);
					int length=strlen(stringOut);
					if (dirtyEscape<numberOfLinesInFile)
						stringOut[length]='\n';
					else
						stringOut[length]='\0';

					stringOut[length+1]='\0';
				}
	
			}
			else
			{
				
				if (stringIn[4]=='S')
				{
					//This line has to be modified
					if (stringIn[1]=='I')
					{
						//This is a Minimum Line
						switch (stringIn[3])
						{
							case 'H':
								if (stringIn[9]=='H')
								{
									sprintf(stringOut,"MINHSWORDH = %d;\n", minHH);
								}
								else
								{
									sprintf(stringOut,"MINHSWORDL = %d;\n", minHL);
								}
							break;
					
							case 'S':
								sprintf(stringOut,"MINSSWORD = %d;\n", minS);
							break;
						
							case 'V':
								sprintf(stringOut,"MINVSWORD = %d;\n", minV);
							break;
						}
					}
					else
					{
						//This is a Maximum Line
						switch (stringIn[3])
						{
							case 'H':
								if (stringIn[9]=='H')
								{
									sprintf(stringOut,"MAXHSWORDH = %d;\n", maxHH);
								}
								else
								{
									sprintf(stringOut,"MAXHSWORDL = %d;\n", maxHL);
								}
							break;
		
							case 'S':
								sprintf(stringOut,"MAXSSWORD = %d;\n", maxS);
							break;
						
							case 'V':
								sprintf(stringOut,"MAXVSWORD = %d;\n", maxV);
							break;
						}
					}
				}
				else
				{	
					//This line goes straight in the output	
					strcpy(stringOut,stringIn);
					int length=strlen(stringOut);
					if (dirtyEscape<numberOfLinesInFile)
						stringOut[length]='\n';
					else
						stringOut[length]='\0';
					stringOut[length+1]='\0';
				}
			}

			fprintf(fout,"%s",stringOut);

	}
	fclose(fout);
	fclose(fin);

	remove("videoconfig.txt");
	rename("videoconfig2.txt","videoconfig.txt");
}

void printResults(int H, int S, int V)
{
    int minHL, maxHL, minHH, maxHH, minV, minS, maxV, maxS;

	minS=(S-40)>0?S-40:0;
	minV=(V-40)>0?V-40:0;
	maxS=(S+40)<255?S+40:255;
	maxV=(V+40)<255?V+40:255;
	minHL=H-15;
	maxHL=H+15;
	maxHH=maxHL;
	minHH=minHL;
	if (maxHH>255)
	{
		minHL=0;
		maxHL=maxHH-255;
		maxHH=255;
	}
	if (minHL<0)
	{
		minHH=minHL+255;
		maxHH=255;
		minHL=0;
	}
    printf("/*These are the computed average values: \n\n"
           "H:%d\n"
           "S:%d\n"
           "V:%d\n\n"
           "We therefore suggest:*/\n\n\n"
           ,H,S,V);
    printf("Robe\n"

           "MINHROBE = %d\n"
           "MAXHROBE = %d\n"
           "MINSROBE = %d\n"
           "MAXSROBE = %d\n"
           "MINVROBE = %d\n"
           "MAXVROBE = %d\n",

           minHH,maxHH,minS, maxS, minV, maxV);
    printf("\n\n\n");

    printf("Sword\n"
           "MINHSWORDL = %d\n"
           "MAXHSWORDL = %d\n"
           "MINHSWORDH = %d\n"
           "MAXHSWORDH = %d\n"
           "MINSSWORD = %d\n"
           "MAXSSWORD = %d\n"
           "MINVSWORD = %d\n"
           "MAXVSWORD = %d\n",
           minHL, maxHL, minHH, maxHH,minS, maxS, minV, maxV);
    printf("\n\n\n\n\n\n\n\n\n\n");
}

void calculate(int* H, int* S, int* V)
{
	int i,j;
	vp_os_mutex_lock(&video_update_lock);
	//char *pixbuf_calc=NULL;
	//pixbuf_calc=vp_os_malloc(boxwidth*boxheight*4*3);
	//vp_os_memset(pixbuf_calc,0,boxwidth*boxheight*4*3);
	
	IplImage* inputRaw=cvCreateImageHeader(cvSize(buffWidth,buffHeight),8,3);
	inputRaw->imageData=pixbuf_data;
	cvSetImageROI(inputRaw,cvRect(buffWidth/2-boxwidth,buffHeight/2-boxheight,2*boxwidth,2*boxheight));

	IplImage* input=cvCreateImage(cvSize(boxwidth*2,boxheight*2),8,3);
	cvCopy(inputRaw,input,NULL);

	/*for(i=buffHeight/2-boxheight; i<buffHeight/2+boxheight; i++)
	{
		for(j=buffWidth/2-boxwidth; j<buffWidth/2+boxwidth; j++)
		{
		pixbuf_calc[(i-(buffHeight/2-boxheight))*3*boxwidth*2+(j-(buffWidth/2-boxwidth))*3+2]=pixbuf_data[j*3+i*buffWidth*3];
		pixbuf_calc[(i-(buffHeight/2-boxheight))*3*boxwidth*2+(j-(buffWidth/2-boxwidth))*3+1]=pixbuf_data[j*3+i*buffWidth*3+1];
		pixbuf_calc[(i-(buffHeight/2-boxheight))*3*boxwidth*2+(j-(buffWidth/2-boxwidth))*3]=pixbuf_data[j*3+i*buffWidth*3+2];
		}
	}*/
	//NOTE: by using cvSetData, the cvReleaseImage won't deallocate the actual imageData buffer because this is considered "external"  cvSetData(imageHeader,dataArray(void*),step);
	//cvSetData(buffer,(void*)imageBuffer,320*3);


	IplImage* hsv=cvCreateImage(cvGetSize(input),8,3);
	cvCvtColor(input,hsv,CV_RGB2HSV);
	
	int height=hsv->height;
	int width=hsv->width;
	int stephsv=hsv->widthStep;
	
	long contatoreH=0;
	long contatoreS=0;
	long contatoreV=0;
	uchar* usdata=(uchar*)hsv->imageData;
	
	for(i=0; i<height; i++)
	{
		for (j=0; j<width; j++)
		{
		contatoreH+=(int)usdata[i*stephsv+j*3]*2*255/360;
		contatoreS+=(int)usdata[i*stephsv+j*3+1];
		contatoreV+=(int)usdata[i*stephsv+j*3+2];
		}
	}
	long divisore=height*width;
	
	*H = (int)(contatoreH/divisore);
	*S = (int)(contatoreS/divisore);
	*V = (int)(contatoreV/divisore);

	cvCvtColor(input,input,CV_RGB2BGR);

	cvShowImage("Selected Area",input);
	
	//cvWaitKey(0);
	cvReleaseImage(&inputRaw);
	cvReleaseImage(&input);
	cvReleaseImage(&hsv);
	
	//vp_os_free(pixbuf_calc);
	vp_os_mutex_unlock(&video_update_lock);
}

gint calculaterobe(GtkWidget *widget, gpointer data)
{
    int H,S,V;

    calculate(&H,&S,&V);
    
    //printResults(H,S,V);
	printResults(H,S,V);
    printResultsOnFile(H,S,V,1);

    return C_OK;
}

gint calculatesword(GtkWidget *widget, gpointer data)
{
    int H,S,V;

    calculate(&H,&S,&V);
    
    //printResults(H,S,V);
	printResults(H,S,V);
    printResultsOnFile(H,S,V,0);

    return C_OK;
}

gboolean update_display(gpointer pData)
{
    update_vision();
    return TRUE;
}
/*User interface main loop*/

DEFINE_THREAD_ROUTINE(userinterface, data)
{
    GtkWidget *outerbox;
    GtkWidget *frowbox;
    GtkWidget *srowbox;
    GtkWidget *buttonbox;;
    GtkWidget *ihbut;
    GtkWidget *dhbut;
    GtkWidget *iwbut;
    GtkWidget *dwbut;
    GtkWidget *calculaterobebut;
    GtkWidget *calculateswordbut;



    gtk_init (NULL, NULL);

    /* Create our window */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

    /* You should always remember to connect the delete_event signal
     * to the main window. This is very important for proper intuitive
     * behavior */
    g_signal_connect (G_OBJECT (window), "delete_event",
                      G_CALLBACK (delete_event), NULL);
    gtk_container_set_border_width (GTK_CONTAINER (window), 10);

    outerbox = gtk_vbox_new (FALSE, 0);
    frowbox = gtk_hbox_new (FALSE, 0);
    srowbox = gtk_hbox_new (FALSE, 0);
    buttonbox = gtk_vbox_new (TRUE, 0);

    gtk_box_pack_start (GTK_BOX (outerbox), frowbox, FALSE, FALSE, 0);

    gtk_box_pack_start (GTK_BOX (outerbox), srowbox, FALSE, FALSE, 0);

    vp_os_mutex_lock(&video_update_lock);

    pixbuf_data=vp_os_malloc(320*240*3);

    vp_os_memset(pixbuf_data,0,320*240*3);

    pixbuf = gdk_pixbuf_new_from_data(pixbuf_data,
                                      GDK_COLORSPACE_RGB,
                                      FALSE,
                                      8,
                                      320,
                                      240,
                                      320*3,
                                      NULL,
                                      NULL);
    image  = (GtkImage*) gtk_image_new_from_pixbuf( pixbuf );

//    gtk_widget_show (image);

    gtk_box_pack_start (GTK_BOX (frowbox), image, FALSE, FALSE, 0);

    vp_os_free(pixbuf_data);

    vp_os_mutex_unlock(&video_update_lock);

    gtk_box_pack_start (GTK_BOX (frowbox), buttonbox, FALSE, FALSE, 0);

    iwbut = gtk_button_new_with_label ("increase width");
    gtk_box_pack_start (GTK_BOX (buttonbox), iwbut, FALSE, FALSE, 0);
    g_signal_connect (G_OBJECT (iwbut), "clicked", G_CALLBACK (incw), NULL);

    dwbut = gtk_button_new_with_label ("decrease width");
    gtk_box_pack_start (GTK_BOX (buttonbox), dwbut, FALSE, FALSE, 0);
    g_signal_connect (G_OBJECT (dwbut), "clicked", G_CALLBACK (decw), NULL);

    ihbut = gtk_button_new_with_label ("increase heigth");
    gtk_box_pack_start (GTK_BOX (buttonbox), ihbut, FALSE, FALSE, 0);
    g_signal_connect (G_OBJECT (ihbut), "clicked", G_CALLBACK (inch), NULL);

    dhbut = gtk_button_new_with_label ("decrease heigth");
    gtk_box_pack_start (GTK_BOX (buttonbox), dhbut, FALSE, FALSE, 0);
    g_signal_connect (G_OBJECT (dhbut), "clicked", G_CALLBACK (dech), NULL);

    calculaterobebut = gtk_button_new_with_label ("calculate robe");
    gtk_box_pack_start (GTK_BOX (buttonbox), calculaterobebut, FALSE, FALSE, 0);
    g_signal_connect (G_OBJECT (calculaterobebut), "clicked", G_CALLBACK (calculaterobe), NULL);

    calculateswordbut = gtk_button_new_with_label ("calculate sword");
    gtk_box_pack_start (GTK_BOX (buttonbox), calculateswordbut, FALSE, FALSE, 0);
    g_signal_connect (G_OBJECT (calculateswordbut), "clicked", G_CALLBACK (calculatesword), NULL);

    infolabel=gtk_label_new( NULL );
    gtk_box_pack_start (GTK_BOX (frowbox) ,infolabel, FALSE, FALSE, 0);
    gtk_container_add (GTK_CONTAINER (window), outerbox);

    gtk_widget_show_all (window);

    g_timeout_add(100, (GtkFunction)update_display, NULL );

    gtk_main();
    return C_OK;
}


/* Implementing thread table in which you add routines of your application and those provided by the SDK */
BEGIN_THREAD_TABLE
THREAD_TABLE_ENTRY( userinterface, 20 )
THREAD_TABLE_ENTRY( ardrone_control, 20 )
THREAD_TABLE_ENTRY( navdata_update, 20 )
THREAD_TABLE_ENTRY( video_stage, 20 )
END_THREAD_TABLE




